/*
 * SerialMousekeyboardControl.cpp - Serial Mouse Keyboard Control V1.0
 * Copyright (C) 20024 Rodrigo Santos - Bit a Bit
 *
 * It is a library that allows you to control the mouse and keyboard
 * using serial communication.
 *For this, it is necessary to use a computer program, responsible for
 *receiving, interpreting the commands and sending the commands to the operating system.
 *
 *Licensed under the MIT License:
 *Permission is hereby granted, free of charge, to any person obtaining a copy
 *of this software and associated documentation files (the "Software"), to deal
 *in the Software without restriction, including without limitation the rights
 *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *copies of the Software, and to permit persons to whom the Software is
 *furnished to do so, subject to the following conditions:
 *
 *The above copyright notice and this permission notice shall be included in all
 *copies or substantial portions of the Software.
 *
 *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *SOFTWARE.
*/

#include "SerialMouseKeyboardControl.h"

SerialMouseKeyboardControl::SerialMouseKeyboardControl( )
{
    left_is_pressed = false;
    right_is_pressed = false;
    middle_is_pressed = false;
}

void
SerialMouseKeyboardControl::
setMousePosition(int x, int y)
{
    String px = String( x );
    String py = String( y );

    String data = String( Device::MOUSE ) + " " + String( MouseEvent::POSITION ) + " " + x + " " + y;

    Serial.println( data );
}

void
SerialMouseKeyboardControl::
mouseMove(int x, int y)
{
    String px = String( x );
    String py = String( y );

    String data = String( Device::MOUSE ) + " " + String( MouseEvent::MOVE ) + " " + x + " " + y;

    Serial.println( data );
}

void
SerialMouseKeyboardControl::
mouseRightPress()
{
    String data = String( Device::MOUSE ) + " " + String( MouseEvent::RIGHT_PRESS );

    Serial.println( data );

    right_is_pressed = true;
}

void
SerialMouseKeyboardControl::
mouseRightRelease()
{

    String data = String( Device::MOUSE ) + " " + String( MouseEvent::RIGHT_RELEASE );

    Serial.println( data );

    right_is_pressed = false;
}

void
SerialMouseKeyboardControl::
mouseLeftPress()
{
    String data = String( Device::MOUSE ) + " " + String( MouseEvent::LEFT_PRESS );

    Serial.println( data );

    left_is_pressed = true;
}

void
SerialMouseKeyboardControl::
mouseLeftRelease()
{
    String data = String( Device::MOUSE ) + " " + String( MouseEvent::LEFT_RELEASE );

    Serial.println( data );

    left_is_pressed = false;
}

void
SerialMouseKeyboardControl::
mouseMiddlePress()
{
    String data =  String( Device::MOUSE ) + " " + String( MouseEvent::MIDDLE_PRESS );

    Serial.println( data );

    middle_is_pressed = true;
}

void
SerialMouseKeyboardControl::
mouseMiddleRelease()
{
    String data = String( Device::MOUSE ) + " " + String( MouseEvent::MIDDLE_RELEASE );

    Serial.println( data );

    middle_is_pressed = false;
}

void
SerialMouseKeyboardControl::
setMouseScroll(int scroll)
{
    String data =  String( Device::MOUSE ) + " " + String( MouseEvent::SCROLL) + " " + String( scroll );

    Serial.println( data );
}

void
SerialMouseKeyboardControl::
keyboardPress(Key key)
{
    String data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::PRESS ) + " " + key;

    Serial.println( data );
}

void
SerialMouseKeyboardControl::
keyboardRelease(Key key)
{
     String data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::RELEASE ) + " " + key;

     Serial.println( data );
}

void
SerialMouseKeyboardControl::
keyboardPress(char key)
{
    int final_key;
    final_key = key;

    String data;

    if( key >= 'A' && key <= 'Z' )
    {
        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::PRESS ) + " " + Key::VK_LSHIFT;
        Serial.println( data );

        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::PRESS ) + " " + final_key;
        Serial.println( data );

        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::RELEASE ) + " " + Key::VK_LSHIFT;
        Serial.println( data );
    }
    else if( key >= 'a' && key <= 'z' )
    {
        final_key = key - 32;

        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::PRESS ) + " " + final_key;
        Serial.println( data );
    }
    else
    {
        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::PRESS ) + " " + final_key;
        Serial.println( data );
    }

}

void
SerialMouseKeyboardControl::
keyboardRelease(char key)
{
    int final_key;
    final_key = key;

    String data;

    if( key >= 'A' && key <= 'Z' )
    {
        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::RELEASE ) + " " + final_key;
        Serial.println( data );

    }
    else if( key >= 'a' && key <= 'z' )
    {
        final_key = key - 32;

        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::RELEASE ) + " " + final_key;
        Serial.println( data );
    }
    else
    {
        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::RELEASE ) + " " + final_key;
        Serial.println( data );
    }
}

void
SerialMouseKeyboardControl::
keyboardPrintln(const char c[])
{
    sendKeys( true, c );
}

void
SerialMouseKeyboardControl::
keyboardPrint(const char c[])
{
    sendKeys( false, c );
}

void
SerialMouseKeyboardControl::
sendKeys(bool new_line, const char str[])
{
    const int size = strlen( str );

    for( int i=0; i<=size; i++ )
    {
        keyboardPress( str[i] );
        keyboardRelease( str[i] );
    }

    if( new_line )
    {
        String data;

        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::PRESS ) + " " + Key::VK_RETURN;
        Serial.println( data );

        data = String( Device::KEYBOARD ) + " " + String( KeyboardEvent::RELEASE ) + " " + Key::VK_RETURN;
        Serial.println( data );
    }
}

